import { registerExtension } from 'vscode/extensions';

const manifest = {"name":"svlangserver","description":"A language server for systemverilog","keywords":["sv","systemverilog","verilog","language","server"],"version":"0.4.1","author":"Kanad Kanhere","publisher":"IMCTradingBV","license":"MIT","repository":{"type":"git","url":"https://github.com/imc-trading/svlangserver.git"},"engines":{"node":"*","vscode":"^1.52.0"},"icon":"resources/icon.png","categories":["Programming Languages","Snippets","Linters","Formatters"],"extensionKind":["workspace"],"bin":{"svlangserver":"./bin/main.js"},"contributes":{"languages":[{"id":"systemverilog","aliases":["SystemVerilog","systemverilog","System Verilog","Systemverilog"],"extensions":[".sv",".svh"],"configuration":"./language-configuration.json"},{"id":"verilog","aliases":["Verilog","verilog"],"extensions":[".v",".vh"],"configuration":"./language-configuration.json"}],"snippets":[{"language":"systemverilog","path":"./snippets/systemverilog.json"},{"language":"verilog","path":"./snippets/systemverilog.json"}],"grammars":[{"language":"systemverilog","scopeName":"source.systemverilog","path":"./syntaxes/systemverilog.tmLanguage.json"},{"language":"verilog","scopeName":"source.systemverilog","path":"./syntaxes/systemverilog.tmLanguage.json"},{"scopeName":"markdown.verilog.codeblock","path":"./syntaxes/codeblock.json","injectTo":["text.html.markdown"],"embeddedLanguages":{"meta.embedded.block.systemverilog":"source.systemverilog","meta.embedded.block.verilog":"source.verilog"}}],"configuration":[{"title":"SystemVerilog Language Server","properties":{"systemverilog.includeIndexing":{"type":"array","default":["**/*.sv"],"description":"Files included for indexing (glob pattern)."},"systemverilog.mustIncludeIndexing":{"type":"array","default":[],"description":"Files included for indexing and are always to be linted (glob pattern)."},"systemverilog.libraryIndexing":{"type":"array","default":[],"description":"Files not included for indexing but needed for linting (glob pattern)."},"systemverilog.excludeIndexing":{"type":"array","default":["**/test/**"],"description":"Files excluded for indexing (glob pattern)."},"systemverilog.linter":{"type":"string","enum":["icarus","verilator"],"default":"verilator","description":"Linter to use"},"systemverilog.launchConfiguration":{"type":"string","default":"verilator -sv --lint-only --Wall","description":"Command used for linting","examples":["verilator -sv --lint-only --Wall","iverilog -t null"]},"systemverilog.defines":{"type":"array","default":[],"description":"Defines needed for linting."},"systemverilog.lintOnUnsaved":{"type":"boolean","default":true,"description":"Enable linting on unsaved files"},"systemverilog.linterWhitelist":{"type":"array","default":["/Unsupported: Interfaced port on top level module/i","/Parent instance's interface is not found/i"],"description":"List of linting errors that need to be whitelisted (javascript regex)"},"systemverilog.formatCommand":{"type":"string","default":"verible-verilog-format","description":"Verible verilog format command"},"systemverilog.disableCompletionProvider":{"type":"boolean","default":false,"description":"Disable auto completion provided by the language server"},"systemverilog.disableHoverProvider":{"type":"boolean","default":false,"description":"Disable hover over help provided by the language server"},"systemverilog.disableSignatureHelpProvider":{"type":"boolean","default":false,"description":"Disable signature help provided by the language server"},"systemverilog.disableLinting":{"type":"boolean","default":false,"description":"Disable verilator linting"}}}],"jsonValidation":[{"fileMatch":"tasks.json","url":"./schemas/tasks.schema.json"}],"problemMatchers":[{"name":"verilator-error","owner":"Verilator","source":"systemverilog","severity":"error","fileLocation":"autoDetect","pattern":[{"regexp":"%Error(-[A-Z0-9]+)?: ((\\S+):(\\d+):((\\d+):)? )?(.*)$","severity":1,"file":2,"line":3,"column":4,"message":5}]},{"name":"verilator-warning","owner":"Verilator","source":"systemverilog","severity":"warning","fileLocation":"autoDetect","pattern":[{"regexp":"%Warning(-[A-Z0-9]+)?: ((\\S+):(\\d+):((\\d+):)? )?(.*)$","severity":1,"file":2,"line":3,"column":4,"message":5}]}]},"files":["/lib/*.js","/lib/*.d.ts","/bin/*.js"]};

const { registerFileUrl, whenReady } = registerExtension(manifest, undefined, {"system":true,"readmePath":"README.md","changelogPath":"CHANGELOG.md"});


registerFileUrl('README.md', new URL('../assets/IMCTradingBV.svlangserver-0.4.1.vsix/README.md', import.meta.url).href, {"mimeType":"text/markdown","size":13153});

registerFileUrl('CHANGELOG.md', new URL('../assets/IMCTradingBV.svlangserver-0.4.1.vsix/CHANGELOG.md', import.meta.url).href, {"mimeType":"text/markdown","size":2509});

registerFileUrl('syntaxes/systemverilog.tmLanguage.json', new URL('../assets/IMCTradingBV.svlangserver-0.4.1.vsix/systemverilog.tmLanguage.json', import.meta.url).href, {"mimeType":"application/json","size":34503});

registerFileUrl('syntaxes/codeblock.json', new URL('../assets/IMCTradingBV.svlangserver-0.4.1.vsix/codeblock.json', import.meta.url).href, {"mimeType":"application/json","size":2693});

registerFileUrl('language-configuration.json', new URL('../assets/IMCTradingBV.svlangserver-0.4.1.vsix/language-configuration.json', import.meta.url).href, {"mimeType":"application/json","size":1723});

registerFileUrl('snippets/systemverilog.json', new URL('../assets/IMCTradingBV.svlangserver-0.4.1.vsix/systemverilog.json', import.meta.url).href, {"mimeType":"application/json","size":9978});

registerFileUrl('schemas/tasks.schema.json', new URL('../assets/IMCTradingBV.svlangserver-0.4.1.vsix/tasks.schema.json', import.meta.url).href, {"mimeType":"application/json","size":1023});

registerFileUrl('package.json', new URL('../assets/IMCTradingBV.svlangserver-0.4.1.vsix/package.json', import.meta.url).href, {"mimeType":"application/json","size":9937});

registerFileUrl('resources/icon.png', new URL('../assets/IMCTradingBV.svlangserver-0.4.1.vsix/icon.png', import.meta.url).href, {"mimeType":"image/png","size":6072});

export { whenReady };
