import { registerExtension } from 'vscode/extensions';

const manifest = {"name":"metals","displayName":"Scala (Metals)","description":"Scala language server with rich IDE features","keywords":["scala","lsp","language server","ide","scalameta"],"version":"1.29.10","publisher":"scalameta","contributors":[{"name":"Alexey Alekhin","url":"https://github.com/laughedelic"},{"name":"Gabriele Petronella","url":"https://github.com/gabro"},{"name":"Ólafur Páll Geirsson","url":"https://github.com/olafurpg"}],"license":"Apache-2.0","bugs":{"url":"https://github.com/scalameta/metals-vscode/issues"},"repository":{"type":"git","url":"https://github.com/scalameta/metals-vscode.git"},"homepage":"https://scalameta.org/metals/","engines":{"vscode":"^1.63.0"},"icon":"images/logo.png","categories":["Programming Languages","Debuggers"],"contributes":{"languages":[{"id":"javap-verbose","extensions":[".javap-verbose"],"icon":{"dark":"icons/file_type_java.svg","light":"icons/file_type_java.svg"}},{"id":"semanticdb","extensions":[".semanticdb-detailed",".semanticdb-compact"]},{"id":"javap","extensions":[".javap"],"icon":{"dark":"icons/file_type_java.svg","light":"icons/file_type_java.svg"}},{"id":"java","extensions":[".cfr"],"icon":{"dark":"icons/file_type_java.svg","light":"icons/file_type_java.svg"}},{"id":"tasty","aliases":["TASTy","tasty"],"extensions":[".tasty-decoded"],"icon":{"dark":"icons/file_type_scala.svg","light":"icons/file_type_scala.svg"}},{"id":"buildtarget","extensions":[".metals-buildtarget"],"icon":{"dark":"icons/book-dark.svg","light":"icons/book-light.svg"}},{"id":"semanticdb","extensions":[".semanticdb",".semanticdb-detailed",".semanticdb-compact",".semanticdb-proto"],"icon":{"dark":"icons/scalameta-logo.png","light":"icons/scalameta-logo.png"}}],"grammars":[{"language":"javap-verbose","scopeName":"source.jvm-bytecode-verbose","path":"./syntaxes/jvm-bytecode-verbose.json"},{"language":"javap","scopeName":"source.jvm-bytecode","path":"./syntaxes/jvm-bytecode.json"},{"language":"tasty","scopeName":"source.tasty","path":"./syntaxes/tasty.json"},{"language":"buildtarget","scopeName":"source.buildtarget","path":"./syntaxes/buildtarget.json"},{"language":"semanticdb","scopeName":"source.semanticdb","path":"./syntaxes/semanticdb.json"}],"configurationDefaults":{"[scala]":{"editor.suggestSelection":"first","editor.formatOnPaste":true,"editor.formatOnType":true}},"configuration":{"title":"Metals","properties":{"metals.serverVersion":{"type":"string","default":"1.2.2","markdownDescription":"The version of the Metals server artifact. Requires reloading the window.  \n\n**VS Code extension version is guaranteed to work only with the default version, change if you know what you're doing**"},"metals.serverProperties":{"type":"array","items":{"type":"string"},"default":["-Xmx1G"],"markdownDescription":"Optional list of properties to pass along to the Metals server. By default, the environment variable `JAVA_OPTS` and `.jvmopts` file are respected. Each property needs to be a separate item.\n\nExample: `-Dhttps.proxyHost=…`, `-Dhttps.proxyPort=…` or `-Dmetals.statistics=all`"},"metals.coursierMirror":{"type":"string","markdownDescription":"Optional alternative repository to maven central if maven central is not available. For example: `https://jcenter.bintray.com`, which will be used to set to property described [here](https://get-coursier.io/blog/#mirrors)"},"metals.ammoniteJvmProperties":{"type":"array","items":{"type":"string"},"markdownDescription":"Optional list of JVM properties to pass along to the Ammonite server. Each property needs to be a separate item.\n\nExample: `-Xmx1G` or `-Xms100M`"},"metals.excludedPackages":{"type":"array","default":[],"markdownDescription":"List of packages you'd like to be left out of completions, symbol searches, and code actions.\n\nEx. `akka.actor.typed.javadsl` will ensure nothing in the `javadsl` package gets recommended to you.\n\nYou can find the list of default exclusions [here on the Metals website](https://scalameta.org/metals/docs/editors/user-configuration/#excluded-packages).\n\nIf you need to remove one of the defaults, you can simply include it and preface it with `--`."},"metals.showInferredType":{"type":"string","default":"false","enum":["false","true","minimal"],"markdownDescription":"When this option is set to true inferred type in all possible places will be shown. This includes types for values, definitions, cases, type parameters. With 'minimal' option Metals will not show inferred type for type parameters and more complicated pattern matches."},"metals.showImplicitArguments":{"type":"boolean","markdownDescription":"When this option is enabled, for each method that has implicit arguments they are displayed as additional decorations."},"metals.showImplicitConversionsAndClasses":{"type":"boolean","markdownDescription":"When this option is enabled, each implicit method and class is displayed as additional decorations at the usage site."},"metals.javaHome":{"type":"string","scope":"machine-overridable","markdownDescription":"Optional path to the Java home directory that will be used for compiling the project.\n\nDefaults to JDK used by Metals's server (look: Java Version).\n\nThis Java version should be lower or equal to JDK version used by the Metals's server."},"metals.javaVersion":{"type":"string","default":"17","enum":["11","17","21"],"scope":"machine","markdownDescription":"JDK version used for running the Metals's server.\n\nShould be greater or equal to the JDK version used for the project (look: Java Home)."},"metals.sbtScript":{"type":"string","markdownDescription":"Optional absolute path to an `sbt` executable to use for running `sbt bloopInstall`.\n\nBy default, Metals uses `java -jar sbt-launch.jar` with an embedded launcher while respecting `.jvmopts` and `.sbtopts`.\n\nUpdate this setting if your `sbt` script requires more customizations like using environment variables."},"metals.millScript":{"type":"string","markdownDescription":"Optional absolute path to a `mill` executable to use for running `mill mill.contrib.Bloop/install`.\n\nBy default, Metals uses an embedded `millw` script while respecting `.mill-version` file.\n\nUpdate this setting if your `mill` script requires more customizations."},"metals.mavenScript":{"type":"string","markdownDescription":"Optional absolute path to a `mvn` executable to use for running `mvn ch.epfl.scala:maven-bloop_2.10:<bloop_version>:bloopInstall`.\n\nBy default, Metals uses an embedded `mvnw` script.\n\nUpdate this setting if your `mvn` script requires more customizations."},"metals.gradleScript":{"type":"string","markdownDescription":"Optional absolute path to a `gradle` executable to use for running `gradle bloopInstall`.\n\nBy default, Metals uses an embedded `gradlew` script.\n\nUpdate this setting if your `gradle` script requires more customizations."},"metals.scalafmtConfigPath":{"type":"string","markdownDescription":"Optional custom path to the .scalafmt.conf file.\n\nShould be an absolute path and use forward slashes `/` for file separators (even on Windows)."},"metals.scalafixConfigPath":{"type":"string","markdownDescription":"Optional custom path to the .scalafix.conf file.\n\nShould be an absolute path and use forward slashes `/` for file separators (even on Windows)."},"metals.scalafixRulesDependencies":{"type":"array","scope":"machine","items":{"type":"string"},"markdownDescription":"List of Scalafix rules dependencies in case they are not available by default in Metals and running scalafix fails due to 'rule not found' exception. For example: `com.github.liancheng::organize-imports:0.6.0` which follows the [coursier](https://get-coursier.io/) convention."},"metals.bloopVersion":{"type":"string","scope":"machine","markdownDescription":"This version will be used for the Bloop build tool plugin, for any supported build tool,while importing in Metals as well as for running the embedded server"},"metals.bloopJvmProperties":{"type":"array","scope":"machine","items":{"type":"string"},"markdownDescription":"Optional list of JVM properties to pass along to the Bloop server. Each property needs to be a separate item.\n\nExample: `-Xmx1G` or `-Xms100M`"},"metals.bloopSbtAlreadyInstalled":{"type":"boolean","markdownDescription":"If true, Metals will not generate a `project/metals.sbt` file under the assumption that sbt-bloop is already manually installed in the sbt build. Build import will fail with a 'not valid command bloopInstall' error in case Bloop is not manually installed in the build when using this option."},"metals.customRepositories":{"type":"array","items":{"type":"string"},"markdownDescription":"Optional list of custom resolvers passed to Coursier when fetching metals dependencies.\n\nFor documentation on accepted values see the [Coursier documentation](https://get-coursier.io/docs/other-repositories).\n\nThe extension will pass these to Coursier using the COURSIER_REPOSITORIES environment variable after joining the custom repositories with a pipe character (|)."},"metals.superMethodLensesEnabled":{"type":"boolean","default":true,"description":"Enable/disable goto super method code lens."},"metals.enableStripMarginOnTypeFormatting":{"type":"boolean","default":true,"markdownDescription":"When enabled, if you press the return key from the first line of a multiline string containing a pipe, it will automatically add `.stripMargin`."},"metals.enableIndentOnPaste":{"type":"boolean","default":false,"markdownDescription":"When this option is enabled, when user pastes any snippet into a Scala file, Metals will try to adjust the indentation to that of the current cursor."},"metals.verboseCompilation":{"type":"boolean","default":false,"markdownDescription":"If a build server supports it (for example Bloop or Scala CLI), setting it to true will make the logs contain all the possible debugging information including that about incremental compilation in Zinc."},"metals.enableSemanticHighlighting":{"type":"boolean","default":true,"markdownDescription":"When this option is enabled, Metals will provide semantic tokens for clients that support it. The feature is still experimental and does not work for all sources."},"metals.testUserInterface":{"type":"string","default":"Test Explorer","enum":["Test Explorer","Code Lenses"],"markdownDescription":"Specifies which UI should be used for tests. `Code lenses` shows virtual text above test class, just like it was previously. `Test explorer` is a new option which shows gutter icons on the left of the file in a less intrusive way. Moreover, this option comes with a new panel called `Testing` which lists all test suites in your codebase and allows you to browse tests and navigate to the file which contains them."},"metals.customProjectRoot":{"type":"string","default":"","scope":"machine-overridable","markdownDescription":"Optional relative path to your project's root. If you want your project root to be the workspace/workspace root set it to \".\" ."},"metals.fallbackScalaVersion":{"type":"string","default":"automatic","enum":["automatic","2.13.12","2.13.11","2.13.10","2.13.9","2.13.8","2.13.7","2.13.6","2.13.5","2.12.18","2.12.17","2.12.16","2.12.15","2.12.14","2.12.13","2.12.12","2.12.11","2.11.12","3.3.1","3.3.0","3.2.2","3.2.1","3.2.0","3.1.3","3.1.2","3.1.1","3.1.0"],"markdownDescription":"The Scala compiler version that is used as the default or fallback in case a file doesn't belong to any build target or the specified Scala version isn't supported by Metals.\n\nThis applies to standalone Scala files, worksheets, and Ammonite scripts.\n\n The `automatic` value means that the Scala version for these files will be inferred from the highest supported Scala version in your projects build definition"},"metals.javaFormat.eclipseConfigPath":{"type":"string","markdownDescription":"Optional custom path to the eclipse-formatter.xml file.\n\nShould be an absolute path and use forward slashes `/` for file separators (even on Windows)."},"metals.javaFormat.eclipseProfile":{"type":"string","markdownDescription":"Optional Eclipse format profile to use."},"metals.scalaCliLauncher":{"type":"string","markdownDescription":"Optional path to Scala CLI launcher to use."},"metals.suggestLatestUpgrade":{"type":"boolean","default":true,"markdownDescription":"Once in a day, notify if there are new server releases (including snapshots)"},"metals.defaultBspToBuildTool":{"type":"boolean","default":false,"markdownDescription":"Default to using build tool as your build server."}}},"submenus":[{"id":"metals.analyze","label":"Metals Analyse Source"}]},"extensionPack":["scala-lang.scala"]};

const { registerFileUrl, whenReady } = registerExtension(manifest, undefined, {"system":true,"readmePath":"README.md"});


registerFileUrl('README.md', new URL('../assets/scalameta.metals-1.29.10.vsix/README.md', import.meta.url).href, {"mimeType":"text/markdown","size":24627});

registerFileUrl('syntaxes/jvm-bytecode-verbose.json', new URL('../assets/scalameta.metals-1.29.10.vsix/jvm-bytecode-verbose.json', import.meta.url).href, {"mimeType":"application/json","size":29887});

registerFileUrl('syntaxes/jvm-bytecode.json', new URL('../assets/scalameta.metals-1.29.10.vsix/jvm-bytecode.json', import.meta.url).href, {"mimeType":"application/json","size":14295});

registerFileUrl('syntaxes/tasty.json', new URL('../assets/scalameta.metals-1.29.10.vsix/tasty.json', import.meta.url).href, {"mimeType":"application/json","size":1277});

registerFileUrl('syntaxes/buildtarget.json', new URL('../assets/scalameta.metals-1.29.10.vsix/buildtarget.json', import.meta.url).href, {"mimeType":"application/json","size":781});

registerFileUrl('syntaxes/semanticdb.json', new URL('../assets/scalameta.metals-1.29.10.vsix/semanticdb.json', import.meta.url).href, {"mimeType":"application/json","size":1123});

registerFileUrl('icons/file_type_java.svg', new URL('../assets/scalameta.metals-1.29.10.vsix/file_type_java.svg', import.meta.url).href, {"mimeType":"image/svg+xml","size":1524});

registerFileUrl('icons/file_type_scala.svg', new URL('../assets/scalameta.metals-1.29.10.vsix/file_type_scala.svg', import.meta.url).href, {"mimeType":"image/svg+xml","size":1566});

registerFileUrl('icons/book-dark.svg', new URL('../assets/scalameta.metals-1.29.10.vsix/book-dark.svg', import.meta.url).href, {"mimeType":"image/svg+xml","size":622});

registerFileUrl('icons/book-light.svg', new URL('../assets/scalameta.metals-1.29.10.vsix/book-light.svg', import.meta.url).href, {"mimeType":"image/svg+xml","size":612});

registerFileUrl('icons/scalameta-logo.png', new URL('../assets/scalameta.metals-1.29.10.vsix/scalameta-logo.png', import.meta.url).href, {"mimeType":"image/png","size":10921});

registerFileUrl('package.json', new URL('../assets/scalameta.metals-1.29.10.vsix/package.json', import.meta.url).href, {"mimeType":"application/json","size":40476});

registerFileUrl('images/logo.png', new URL('../assets/scalameta.metals-1.29.10.vsix/logo.png', import.meta.url).href, {"mimeType":"image/png","size":6790});

export { whenReady };
